import sys

try:
    from gi.repository import Gtk
    got_gobject_introspection = True
except ImportError:
    got_gobject_introspection = False
    gobject_skip_message = "GObject Introspection module unavailable"

from landscape.tests.helpers import LandscapeTest
from landscape.configuration import LandscapeSetupConfiguration
from landscape.ui.view.configuration import ClientSettingsDialog
from landscape.ui.controller.configuration import ConfigController


class ConfigurationViewTest(LandscapeTest):

    def setUp(self):
        super(ConfigurationViewTest, self).setUp()
        config = "[client]\n"
        config += "data_path = %s\n" % sys.path[0]
        config += "http_proxy = http://proxy.localdomain:3192\n"
        config += "tags = a_tag\n"
        config += "url = https://landscape.canonical.com/message-system\n"
        config += "account_name = foo\n"
        config += "registration_password = bar\n"
        config += "computer_title = baz\n"
        config += "https_proxy = https://proxy.localdomain:6192\n"
        config += "ping_url = http://landscape.canonical.com/ping\n"

        self.config_filename = self.makeFile(config)

        class MySetupConfiguration(LandscapeSetupConfiguration):
            default_config_filenames = [self.config_filename]

        self.config = MySetupConfiguration()

    def test_init(self):
        """
        Test that we correctly initialise the L{ConfigurationView} correctly
        from the controller.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        content_area = dialog.get_content_area()
        children = content_area.get_children()
        self.assertEqual(len(children), 2)
        box = children[0]
        self.assertIsInstance(box, Gtk.Box)
        self.assertTrue(dialog._hosted_radiobutton.get_active())
        self.assertFalse(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._account_entry.get_sensitive())
        self.assertTrue(dialog._password_entry.get_sensitive())
        self.assertFalse(dialog._server_host_name_entry.get_sensitive())
        self.assertTrue(dialog._registration_button.get_sensitive())
        start, end = dialog._registration_textbuffer.get_bounds()
        self.assertEqual(
            len(dialog._registration_textbuffer.get_text(
                    start, end, include_hidden_chars=True)),
            0)

    def test_toggle_radio_button(self):
        """
        Test that we disable and enable the correct entries when we toggle the
        dialog radiobuttons.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        self.assertTrue(dialog._hosted_radiobutton.get_active())
        self.assertFalse(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._account_entry.get_sensitive())
        self.assertTrue(dialog._password_entry.get_sensitive())
        self.assertFalse(dialog._server_host_name_entry.get_sensitive())
        dialog._dedicated_radiobutton.set_active(True)
        self.assertFalse(dialog._hosted_radiobutton.get_active())
        self.assertTrue(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._account_entry.get_sensitive())
        self.assertTrue(dialog._password_entry.get_sensitive())
        self.assertTrue(dialog._server_host_name_entry.get_sensitive())
        dialog._hosted_radiobutton.set_active(True)
        self.assertTrue(dialog._hosted_radiobutton.get_active())
        self.assertFalse(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._account_entry.get_sensitive())
        self.assertTrue(dialog._password_entry.get_sensitive())
        self.assertFalse(dialog._server_host_name_entry.get_sensitive())

    def test_load_data_from_config(self):
        """
        Test that we load data into the appropriate entries from the
        configuration file.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        self.assertEqual(dialog._account_entry.get_text(), "foo")
        self.assertEqual(dialog._password_entry.get_text(), "bar")
        self.assertEqual(dialog._server_host_name_entry.get_text(),
                         "landscape.canonical.com")

    def test_modification(self):
        """
        Test that modifications to the dialog cause the close button to be
        disabled and the connect button to be reset.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        self.assertEqual(dialog._registration_image.get_stock(),
                         (Gtk.STOCK_DISCONNECT, 4))
        dialog._registration_image.set_from_stock(Gtk.STOCK_CONNECT, 4)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.assertEqual(dialog._registration_image.get_stock(),
                         (Gtk.STOCK_CONNECT, 4))
        dialog._account_entry.set_text("Spangles!")
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.assertEqual(dialog._registration_image.get_stock(),
                         (Gtk.STOCK_DISCONNECT, 4))
        self.assertFalse(dialog._close_button.get_sensitive())

    def test_revert(self):
        """
        Test that we can revert the UI values using the controller.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        self.assertEqual(dialog._account_entry.get_text(), "foo")
        self.assertEqual(dialog._password_entry.get_text(), "bar")
        self.assertEqual(dialog._server_host_name_entry.get_text(),
                         "landscape.canonical.com")
        dialog._dedicated_radiobutton.set_active(True)
        dialog._server_host_name_entry.set_text("more.barn")
        self.assertEqual(dialog._account_entry.get_text(), "standalone")
        self.assertEqual(dialog._password_entry.get_text(), "bar")
        self.assertEqual(dialog._server_host_name_entry.get_text(),
                         "more.barn")
        self.assertTrue(dialog._dedicated_radiobutton.get_active())
        self.assertFalse(dialog._hosted_radiobutton.get_active())
        dialog.revert(None)
        self.assertEqual(dialog._account_entry.get_text(), "foo")
        self.assertEqual(dialog._password_entry.get_text(), "bar")
        self.assertEqual(dialog._server_host_name_entry.get_text(),
                         "landscape.canonical.com")
        self.assertFalse(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._hosted_radiobutton.get_active())

    if not got_gobject_introspection:
        test_revert.skip = gobject_skip_message
        test_load_data_from_config.skip = gobject_skip_message
        test_toggle_radio_button.skip = gobject_skip_message
        test_init.skip = gobject_skip_message


class ConfigurationViewCommitTest(LandscapeTest):

    def setUp(self):
        super(ConfigurationViewCommitTest, self).setUp()
        config = "[client]\n"
        config += "data_path = %s\n" % sys.path[0]
        config += "http_proxy = http://proxy.localdomain:3192\n"
        config += "tags = a_tag\n"
        config += "url = https://landscape.canonical.com/message-system\n"
        config += "account_name = foo\n"
        config += "registration_password = bar\n"
        config += "computer_title = baz\n"
        config += "https_proxy = https://proxy.localdomain:6192\n"
        config += "ping_url = http://landscape.canonical.com/ping\n"
        self.config_filename = self.makeFile(config)

        class MySetupConfiguration(LandscapeSetupConfiguration):
            default_config_filenames = [self.config_filename]

        self.config = MySetupConfiguration()
        self.real_write_back = ClientSettingsDialog._write_back
        self.write_back_called = False

        def fake_write_back(obj):
            self.write_back_called = True

        ClientSettingsDialog._write_back = fake_write_back
        self.controller = ConfigController(self.config)
        self.dialog = ClientSettingsDialog(self.controller)

    def tearDown(self):
        self.controller = None
        self.dialog.destroy()
        self.dialog = None
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.write_back_called = False
        ClientSettingsDialog._write_back = self.real_write_back
        super(ConfigurationViewCommitTest, self).tearDown()

    def test_commit_fresh_dialog(self):
        """
        Test that we don't save anything from an untouched dialog on exit
        """
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.dialog._possibly_save_and_exit(None)
        self.assertFalse(self.write_back_called)

    def test_commit_hosted_account_name_change(self):
        """
        Test that we do save changes when we set a new account name for hosted
        accounts.
        """
        self.dialog._hosted_radiobutton.set_active(True)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.dialog._account_entry.set_text("glow")
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.dialog._possibly_save_and_exit(None)
        self.assertTrue(self.write_back_called)

    def test_commit_hosted_password_change(self):
        """
        Test that we do save changes when we set a new account name for hosted
        accounts.
        """
        self.dialog._hosted_radiobutton.set_active(True)
        self.dialog._password_entry.set_text("sticks")
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.dialog._possibly_save_and_exit(None)
        self.assertTrue(self.write_back_called)

    def test_commit_dedicated_server_host_name_change(self):
        """
        Test that we do save changes when we set a new server host name for
        a dedicated server.
        """
        self.dialog._dedicated_radiobutton.set_active(True)
        self.dialog._server_host_name_entry.set_text(
            "that.isolated.geographic")
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.dialog._possibly_save_and_exit(None)
        self.assertTrue(self.write_back_called)

    if not got_gobject_introspection:
        test_commit_dedicated_server_host_name_change.skip = \
            gobject_skip_message
        test_commit_hosted_password_change.skip = gobject_skip_message
        test_commit_hosted_account_name_change.skip = gobject_skip_message
        test_commit_fresh_dialog.skip = gobject_skip_message


class DedicatedConfigurationViewTest(LandscapeTest):

    def setUp(self):
        super(DedicatedConfigurationViewTest, self).setUp()
        config = "[client]\n"
        config += "data_path = %s\n" % sys.path[0]
        config += "url = https://landscape.localdomain/message-system\n"
        config += "computer_title = baz\n"
        config += "ping_url = http://landscape.localdomain/ping\n"
        self.config_filename = self.makeFile(config)

        class MySetupConfiguration(LandscapeSetupConfiguration):
            default_config_filenames = [self.config_filename]

        self.config = MySetupConfiguration()

    def test_init(self):
        """
        Test that we correctly initialise the L{ConfigurationView} correctly
        from the controller.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        content_area = dialog.get_content_area()
        children = content_area.get_children()
        self.assertEqual(len(children), 2)
        box = children[0]
        self.assertIsInstance(box, Gtk.Box)
        self.assertFalse(dialog._hosted_radiobutton.get_active())
        self.assertTrue(dialog._dedicated_radiobutton.get_active())
        self.assertTrue(dialog._account_entry.get_sensitive())
        self.assertTrue(dialog._password_entry.get_sensitive())
        self.assertTrue(dialog._server_host_name_entry.get_sensitive())

    def test_load_data_from_config(self):
        """
        Test that we load data into the appropriate entries from the
        configuration file.
        """
        controller = ConfigController(self.config)
        dialog = ClientSettingsDialog(controller)
        self.assertEqual(dialog._account_entry.get_text(), "")
        self.assertEqual(dialog._password_entry.get_text(), "")
        self.assertEqual(dialog._server_host_name_entry.get_text(),
                         "landscape.localdomain")

    if not got_gobject_introspection:
        test_load_data_from_config.skip = gobject_skip_message
        test_init.skip = gobject_skip_message
